#!/bin/bash

# Copyright (c) 2014 Mellanox Technologies. All rights reserved.
#
# This Software is licensed under one of the following licenses:
#
# 1) under the terms of the "Common Public License 1.0" a copy of which is
#    available from the Open Source Initiative, see
#    http://www.opensource.org/licenses/cpl.php.
#
# 2) under the terms of the "The BSD License" a copy of which is
#    available from the Open Source Initiative, see
#    http://www.opensource.org/licenses/bsd-license.php.
#
# 3) under the terms of the "GNU General Public License (GPL) Version 2" a
#    copy of which is available from the Open Source Initiative, see
#    http://www.opensource.org/licenses/gpl-license.php.
#
# Licensee has the right to choose one of the above licenses.
#
# Redistributions of source code must retain the above copyright
# notice and one of the license notices.
#
# Redistributions in binary form must reproduce both the above copyright
# notice, one of the license notices in the documentation
# and/or other materials provided with the distribution.

VERSION="1.2"

rpmname="mlnxofed"
version=
release=
mlnx_ofed_dir=
conf=
verbose=0

TMP=/tmp
OUTDIR=$PWD
distro=cust
arch=$(uname -m)

###########

usage()
{
cat << EOF

	Usage: `basename $0` [-m|--mlnx_ofed] <path> [-c|--conf] <path> [options]

	Options:
		[-m|--mlnx_ofed]                     path to MLNX_OFED directory.
		[-c|--conf]                          path to packages config file.
		                                      - Syntax: <package>=y
		                                      - Lines starting with '#', or that have <package>=n are ignored.
		[-n|--rpm_name]                      Set package name to create (Default: $rpmname).
		[--rpm-version]                      Set package major version number (Default: MLNX_OFED major version number).
		[--rpm-release]                      Set package release version number (Default: MLNX_OFED release version number).
		[--noarch]                           Build noarch package (Default: $arch).
		[-o|--output-dir]                    Output directory (Default: $OUTDIR).
		[-t|--tmpdir]                        Local work directory (Default: $TMP).
		[-v|--verbose]                       Verbose.
		--version                            Print tool version.

EOF
}

if [ $UID -ne 0 ]; then
    echo You must be root to run `basename $0`
    exit 1
fi

if [ -z "$1" ]; then
	usage
	exit 1
fi

while [ ! -z "$1" ]
do
	case "$1" in
		-h | --help)
		usage
		shift
		exit 0
		;;
		-m | --mlnx_ofed)
		mlnx_ofed_dir=$2
		shift 2
		;;
		-c | --conf)
		conf=$2
		shift 2
		;;
		-n | --rpm_name)
		rpmname=$2
		shift 2
		;;
		-v | --verbose)
		verbose=1
		shift
		;;
		-o | --output-dir)
		OUTDIR=$2
		shift 2
		;;
		-t | --tmpdir)
		TMP=$2
		shift 2
		;;
		--version)
		echo "Version: $VERSION"
		exit 0
		;;
		--rpm-version)
		version=$2
		shift 2
		;;
		--rpm-release)
		release=$2
		shift 2
		;;
		--noarch)
		arch=noarch
		shift
		;;
		*)
		usage
		shift
		exit 1
		;;
	esac
done

if [ -z "$mlnx_ofed_dir" ] || [ ! -d "$mlnx_ofed_dir" ]; then
	echo "Path to MLNX_OFED directory is not defined or does not exist!"
	usage
	exit 1
fi

if [[ ! -f $mlnx_ofed_dir/.mlnx && ! -f $mlnx_ofed_dir/mlnx ]]; then
	echo "$mlnx_ofed_dir is not a supported MLNX_OFED_LINUX directory"
	exit 1
fi

if [ -z "$conf" ] || [ ! -f "$conf" ]; then
	echo "Path to packages conf file is not defined or does not exist!"
	usage
	exit 1
fi

# get MLNX_OFED version
ofed_version=
ofed_release=
if [ -f $mlnx_ofed_dir/.mlnx ]; then
	ofed_version=`cat $mlnx_ofed_dir/.mlnx | cut -d'-' -f1`
	ofed_release=`cat $mlnx_ofed_dir/.mlnx | cut -d'-' -f2`
else
	ofed_version=`cat $mlnx_ofed_dir/mlnx | cut -d'-' -f1`
	ofed_release=`cat $mlnx_ofed_dir/mlnx | cut -d'-' -f2`
fi

if [ -z "$version" ]; then
	version=$ofed_version
fi
if [ -z "$release" ]; then
	release=$ofed_release
fi

if [ -z "$version" ]; then
	echo "version is not defined!"
	usage
	exit 1
fi

if [ -z "$release" ]; then
	echo "release is not defined!"
	usage
	exit 1
fi

echo Detected MLNX_OFED version: ${ofed_version}-${ofed_release}

if [ ! -d "$OUTDIR" ]; then
	mkdir -p $OUTDIR
	if [ $? -ne 0 ]; then
		echo "-E- Can't create output dir '$OUTDIR'"
		exit 1
	fi
fi
WDIR=`mktemp -d /${TMP}/mlnxofed.XXXXXXX`
topdir=${WDIR}/topdir.${distro}.${arch}
/bin/rm -rf $topdir
mkdir -p $topdir/{SOURCES,SPECS,BUILD,SRPMS,RPMS}
mkdir -p $topdir/SOURCES/${rpmname}-${version}
echo ${rpmname}-${version} > $topdir/SOURCES/${rpmname}-${version}/${rpmname}-release
pushd $topdir/SOURCES > /dev/null
tar czf ${rpmname}-${version}.tar.gz ${rpmname}-${version}
popd > /dev/null

#######
if [ $verbose -eq 1 ]; then
	echo Creating spec file: $topdir/SPECS/${rpmname}.spec
fi

cat > $topdir/SPECS/${rpmname}.spec <<EOF
# spec file
Name: ${rpmname}
Version: ${version}
Release: ${release}
Summary: MLNX_OFED support package
Group: System Environment/Libraries
License: GPLv2 or BSD
Url: http://mellanox.com
Vendor: Mellanox Technologies
Source: http://mellanox.com/%{name}-%{version}.tar.gz
BuildRoot: %(mktemp -ud %{_tmppath}/%{name}-%{version}-%{release}-XXXXXX)
Obsoletes: libmverbs-headers
Obsoletes: ibutils-libs
Obsoletes: mvapich
Obsoletes: mvapich2
Obsoletes: openmpi
Obsoletes: mpitests-mvapich
Obsoletes: mpitests-mvapich2
Obsoletes: mpitests-openmpi
Obsoletes: compat-opensm-libs
Obsoletes: rdma
@OBSOLETES@
@REQUIRES@ 

%description
MLNX_OFED support package

%prep
%setup

%install
install -d \$RPM_BUILD_ROOT/%{_docdir}/%{name}
cp %{name}-release \$RPM_BUILD_ROOT/%{_docdir}/%{name}

%clean
rm -rf \$RPM_BUILD_ROOT

%post
cd /lib/modules
for dd in \`/bin/ls\`
do
	/sbin/depmod \$dd
done

if [ -f /usr/bin/ofed_info ]; then
	sed -i -e "s/OFED-internal/MLNX_OFED_LINUX/g" /usr/bin/ofed_info
fi

%files
%defattr(-,root,root,-)
%{_docdir}

%changelog
EOF
### End of spec file

# add required packages list to spec file
for ff in `find ${mlnx_ofed_dir}/ -name "*.rpm"`
do
	case "$ff" in
		*i686.rpm|*infiniband-diags-guest-*|*compat-dapl*|*el6uek*|*.el5*|*.el6*|*.el7*|*opensm-[0-9]*|*mlnxofed-[0-9]*)
		if [ $verbose -eq 1 ]; then
			echo skipping $ff
		fi
		continue
		;;
	esac

	ff_name=`rpm --nosignature -qp --queryformat "[%{NAME}]" $ff`
	ff_version=`rpm --nosignature -qp --queryformat "[%{VERSION}]-[%{RELEASE}]" $ff`

	# Is the package selected in the conf file ?
	if ! (grep -qwE "^$ff_name=y" $conf); then
		if [ $verbose -eq 1 ]; then
			echo skipping $ff
		fi
		continue
	fi

	if (grep -qw "Requires: $ff_name =" $topdir/SPECS/${rpmname}.spec); then
		continue
	fi

	if [ $verbose -eq 1 ]; then
		echo Adding package: $ff
	fi

	perl -ni -e  "if (m/@REQUIRES@/) {
			print q/@REQUIRES@
Requires: $ff_name = $ff_version
/;
		      } else {print;}" $topdir/SPECS/${rpmname}.spec

done

perl -ni -e  'print unless m/@OBSOLETES@/' $topdir/SPECS/${rpmname}.spec
perl -ni -e  'print unless m/@REQUIRES@/' $topdir/SPECS/${rpmname}.spec
if [ $verbose -eq 1 ]; then
	rpmbuild --target $arch -ba --define "_topdir $topdir" --define "_sourcedir %{_topdir}/SOURCES" --define "_specdir %{_topdir}/SPECS" --define "_srcrpmdir %{_topdir}/SRPMS" --define "_rpmdir %{_topdir}/RPMS" $topdir/SPECS/${rpmname}.spec
else
	rpmbuild --target $arch -ba --define "_topdir $topdir" --define "_sourcedir %{_topdir}/SOURCES" --define "_specdir %{_topdir}/SPECS" --define "_srcrpmdir %{_topdir}/SRPMS" --define "_rpmdir %{_topdir}/RPMS" $topdir/SPECS/${rpmname}.spec > /dev/null 2>&1
fi
if [ $? -ne 0 ]; then
	echo "-E- Failed to build binary rpm!"
	exit 1
fi
bins=`find  $topdir/RPMS/$arch/ -name "*rpm" -exec basename {} \;`
if [ -z "$bins" ]; then
	echo "-E- Couldn't find the built binary rpms!"
	exit 1
else
	cp $topdir/RPMS/$arch/${rpmname}* $OUTDIR
	if [ $? -ne 0 ]; then
		echo "-E- Can't copy '$bins' to '$OUTDIR'"
		exit 1
	fi
	/bin/rm -rf $WDIR
	echo "Built $OUTDIR/$bins"
fi
